"""
KEGG Brite

"""
from __future__ import absolute_import

import os
import io
import re

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from . import conf


class BriteEntry(object):
    _search_re = {
        "ids": re.compile('(?P<ids>\[.*:.*\])'),
        "title": re.compile(r'(<[Bb]>)?(?P<title>\b[a-zA-Z0-9_/\s,;:.+=\-\[\]{}\(\)]+?)(?(1)</[Bb]>)$'),
        "links": re.compile('(?P<links><a href=".+?">.*?</a>)')
    }

    def __init__(self, line):
        self.entries = []
        self.line = line[1:].strip()
        for name, re in self._search_re.items():
            search = re.search(self.line)
            setattr(self, name, search.group(name) if search else None)

    def __iter__(self):
        return iter(self.entries)


class Brite(BriteEntry):
    VERSION = "v1.0"
    BRITE_URL_FORMAT = "http://www.genome.jp/kegg-bin/download_htext?htext={brite_id}.keg&format=htext&filedir="

    def __init__(self, brite_id, local_cache=None):
        super(Brite, self).__init__("")
        self.brite_id = id
        if local_cache is None:
            local_cache = conf.params["cache.path"]
        self.local_cache = local_cache

        self.load(brite_id)

    def _get_brite(self, brite_id):
        url = self.BRITE_URL_FORMAT.format(brite_id=brite_id)
        local_filename = os.path.join(self.local_cache, brite_id + ".keg")
        if not os.path.exists(local_filename):
            brite = urlopen(url).read()
            with io.open(local_filename, "wb") as f:
                f.write(brite)

        return io.open(local_filename, "r", )

    def load(self, brite_id):
        lines = self._get_brite(brite_id).read().split("\n!\n")[1].splitlines()

        # TODO: Implement a proper parser

        def collect(lines, depth, collection):
            while lines:
                line = lines[0]
                if line.startswith("#"):
                    lines.pop(0)
                elif line.startswith(depth) and len(line.strip()) > 1:
                    collection.append(BriteEntry(lines.pop(0)))
                elif line[0] > depth:
                    collect(lines, line[0], collection[-1].entries)
                elif line[0] < depth:
                    return
                else:
                    lines.pop(0)

        collect([line for line in lines
                 if not line.startswith("#") and len(line) > 1], "A",
                self.entries)



""" NCBI Gene Information data update """
import sqlite3
import gzip
import bz2


from server_update import *
from server_update.tests.test_GeneInfo import GeneInfo
from orangecontrib.bioinformatics.ncbi.gene import (
    DOMAIN, FILENAME, FTP_FILENAME, FTP_URL, TITLE, TAGS
)
from orangecontrib.bioinformatics.dicty.config import DICTY_INFO_URL, DICTY_MAPPING_URL
from orangecontrib.bioinformatics.ncbi.taxonomy import common_taxids
from orangecontrib.bioinformatics.ncbi.taxonomy.utils import Taxonomy


# columns indexes
# ftp://ftp.ncbi.nlm.nih.gov/gene/README under "gene_info" section
tax_id, gene_id, symbol, synonyms, db_refs, description = 0, 1, 2, 4, 5, 8
locus_tag, chromosome, map_location, type_of_gene, modification_date = 3, 6, 7, 9, 14
symbol_from_nomenclature_authority, full_name_from_nomenclature_authority = 10, 11
nomenclature_status, other_designations = 12, 13

domain_path = sf_local.localpath(DOMAIN)
temp_path = os.path.join(domain_path, sf_temp)
file_path = os.path.join(domain_path, FILENAME)

create_folder(domain_path)
create_folder(temp_path)
parent_tax_ids = common_taxids()

# we must include all strains of organism. Genes refer to specific strain of an organism
tax_ids = []
map_strain_to_species = {}
tax_obj = Taxonomy()
for parent_id in parent_tax_ids:
    strains = tax_obj.get_all_strains(parent_id)
    tax_ids.append(parent_id)

    if strains:
        [tax_ids.append(strain_id) for strain_id in strains]


init_table = """ 
 CREATE TABLE "gene_info" ( 
    species INTEGER NOT NULL,
    tax_id INTEGER NOT NULL, 
    gene_id INTEGER NOT NULL UNIQUE, 
    symbol INTEGER NOT NULL,
    synonyms TEXT,
    db_refs TEXT,
    description TEXT,
    locus_tag TEXT,
    chromosome TEXT,
    map_location TEXT,
    type_of_gene TEXT,
    symbol_from_nomenclature_authority TEXT,
    full_name_from_nomenclature_authority TEXT,
    nomenclature_status TEXT,
    other_designations TEXT,
    modification_date TEXT,
    PRIMARY KEY(`tax_id`,`gene_id`));

 CREATE TABLE "gene_source" ( 
    tax_id INTEGER NOT NULL, 
    gene_id INTEGER NOT NULL, 
    source TEXT, 
    source_id TEXT,
    FOREIGN KEY (tax_id, gene_id) REFERENCES gene_info(tax_id, gene_id) ON DELETE CASCADE ON UPDATE NO ACTION

  )
"""

gene_info_lines = []
gene_source_lines = []
gene_names_dictyBase = []

# DOWNLOAD FILES
print("Downloading ncbi file ...")
stream = urlopen(FTP_URL, timeout=30)
with open(os.path.join(domain_path, FTP_FILENAME), 'wb') as f:
    shutil.copyfileobj(stream, f)

print("Downloading dictyBase file ...")
stream = urlopen(DICTY_INFO_URL, timeout=30)
with open(os.path.join(domain_path, 'DictyBase'), 'wb') as f:
    shutil.copyfileobj(stream, f)

print("Downloading dictyMappings file ...")
stream = urlopen(DICTY_MAPPING_URL, timeout=30)
with open(os.path.join(domain_path, 'DictyMapping'), 'wb') as f:
    shutil.copyfileobj(stream, f)


# PARSE DICTY BASE FILE
dicty_id, dicty_name = 0, 1
dictyBase_map = dict()

with open(os.path.join(domain_path, 'DictyBase'), 'r') as f:
    f.readline()  # skip header
    for line in f:
        split_line = line.strip().split('\t')
        dictyBase_map[split_line[dicty_id]] = split_line[dicty_name]


# PARSE DICTY MAPPINGS FILE
ddb_id,	ddb_g_id, name, uniProt_id = 0, 1, 2, 3
dictyMapping_map = dict()

with open(os.path.join(domain_path, 'DictyMapping'), 'r') as f:
    f.readline()  # skip header
    for line in f:
        split_line = line.strip().split('\t')
        try:
            dictyMapping_map[split_line[ddb_g_id]] = split_line[uniProt_id]
        except IndexError:
            # no uniprot id for this gene
            pass


# PARSE GENE_INFO FILE
def parse_refs(gene):
    external_ids = gene[db_refs].split('|')

    for ref in external_ids:
        source, source_id = ref.split(':', 1)
        # gene_source_lines.append([int(gene[tax_id]), int(gene[gene_id]), source, source_id])

        if source == 'dictyBase':
            try:
                # Update gene symbol column, we want to have gene symbols from source database for dictyBase
                gene_names_dictyBase.append([int(gene[gene_id]), dictyBase_map[source_id]])

                # add uniProt ID as a source to dicty genes
                if dictyMapping_map[source_id] is not None:
                    gene[db_refs] = gene[db_refs] + '|UniProt:' + dictyMapping_map[source_id]

            except KeyError:
                # dictyBase map is constructed from official database source for dicty.
                # if source_id in ncbi database is not found in that map, it is probably deprecated.
                pass


print("Parsing NCBI gene info file ...")
with gzip.open(os.path.join(domain_path, FTP_FILENAME), 'rb') as info_file:
    info_file.readline()

    for line in info_file:
        split_line = line.decode().split('\t')

        if split_line[0] in tax_ids:
            gene_synonyms = split_line[synonyms]
            split_line[synonyms] = '|' + gene_synonyms + '|' if gene_synonyms != '-' else ''
            gene_info_lines.append(split_line)

            if split_line[db_refs] != '-':
                parse_refs(split_line)


# CREATE SQLITE DB
con = sqlite3.connect(file_path, timeout=15)
cursor = con.cursor()

for table in ["gene_info", "gene_source"]:
    cursor.execute("DROP TABLE IF EXISTS %s" % table)

cursor.executescript(init_table)

cursor.executemany("INSERT INTO gene_info VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                   ((int(tax_obj.get_species(gene[tax_id])),
                     int(gene[tax_id]),
                     int(gene[gene_id]),
                     gene[symbol],
                     gene[synonyms],
                     gene[db_refs],
                     gene[description],
                     gene[locus_tag],
                     gene[chromosome],
                     gene[map_location],
                     gene[type_of_gene],
                     gene[symbol_from_nomenclature_authority],
                     gene[full_name_from_nomenclature_authority],
                     gene[nomenclature_status],
                     gene[other_designations],
                     gene[modification_date])
                    for gene in gene_info_lines))

# cursor.executemany("INSERT INTO gene_source VALUES (?, ?, ?, ?)",
# ([column for column in match] for match in gene_source_lines))

con.commit()

cursor.executemany("UPDATE gene_info SET symbol = ? WHERE gene_id = ?",
                   ([dicty[1], dicty[0]] for dicty in gene_names_dictyBase))

con.commit()
con.close()


# PREPARE SERVERFILES
print("Creating serverfiles ...")
db_size = os.stat(file_path).st_size  # store uncompressed database size

with bz2.BZ2File(os.path.join(temp_path, FILENAME), mode='w', compresslevel=9) as f:
    shutil.copyfileobj(open(os.path.join(domain_path, FILENAME), 'rb'), f)

create_info_file(os.path.join(temp_path, FILENAME),
                 domain=DOMAIN,
                 filename=FILENAME,
                 source=SOURCE_SERVER,
                 title=TITLE,
                 tags=TAGS,
                 uncompressed=db_size,
                 compression='bz2')

helper = SyncHelper(DOMAIN, GeneInfo)

# sync files with remote server
helper.run_tests()
helper.sync_files()

helper.remove_update_folder()



from unittest import TestCase
from server_update import sf_local
from orangecontrib.bioinformatics.geneset.utils import filename_parse


class GeneSetsTest(TestCase):

    def setUp(self):
        self.files = [fn for domain, fn in sf_local.listfiles('gene_sets') if fn != 'index.pck']

    def test_gene_sets(self):
        for file in self.files:
            hierarchy, organism = filename_parse(file)
            # TODO: fix tests
            # load_serverfiles(hierarchy, organism)



import unittest
import os

from tempfile import mkstemp
from orangecontrib.bioinformatics.geneset import GeneSet, GeneSets, filename, filename_parse, GeneSetException


class TestGeneSets(unittest.TestCase):
    test_gs_id = 'test_gs'
    test_name = 'test_name'
    test_organism = '9606'
    test_hierarchy = ('GO', 'biological_process')
    test_file = 'GO-biological_process-9606.gmt'
    test_genes = [123, 321, 111, 222, 333]  # ncbi-like ids

    def test_file_name(self):
        file_name = filename(self.test_hierarchy, self.test_organism)
        self.assertEqual(file_name, self.test_file)

        hierarchy, org = filename_parse(file_name)
        self.assertEqual(hierarchy, self.test_hierarchy)
        self.assertEqual(org, self.test_organism)

    def test_gmt_file_format(self):

        gs = GeneSet(gs_id=self.test_gs_id,
                     name=self.test_name,
                     genes=self.test_genes,
                     hierarchy=self.test_hierarchy,
                     organism=self.test_organism,
                     link='')

        fd, file_name = mkstemp()

        # write to file
        write_sets = GeneSets([gs])
        write_sets.to_gmt_file_format(file_name)

        with open(file_name, 'r') as temp_f:
            line = temp_f.readline()
            columns = line.strip().split('\t')
            self.assertGreater(len(columns), 0)

        # read from file
        read_sets = GeneSets.from_gmt_file_format(file_name)
        self.assertIsNotNone(read_sets)
        self.assertGreater(len(read_sets), 0)
        self.assertEqual(read_sets.common_hierarchy(), self.test_hierarchy)
        self.assertEqual(read_sets.common_org(), self.test_organism)

        # clean-up
        os.close(fd)
        os.remove(file_name)

    def test_gene_set(self):

        gs1 = GeneSet(gs_id=self.test_gs_id,
                      name=self.test_name,
                      genes=self.test_genes,
                      hierarchy=self.test_hierarchy,
                      organism=self.test_organism,
                      link='')

        gs2 = GeneSet(gs_id='test2', name='test_name2')

        self.assertEqual(gs1.gmt_description(), 'test_gs,GO-biological_process,9606,test_name,_,_,_')
        self.assertFalse(gs1 == gs2)
        self.assertNotEqual(gs1, gs2)
        self.assertTrue(gs1 == gs1)

    def test_gene_sets(self):
        gs1 = GeneSet(gs_id=self.test_gs_id,
                      name=self.test_name,
                      genes=self.test_genes,
                      hierarchy=self.test_hierarchy,
                      organism=self.test_organism,
                      link='')

        gs2 = GeneSet(gs_id='test2', name='test_name2', hierarchy=('Test', 'test'), organism='3702')
        gs3 = GeneSet(gs_id='test3', name='test_name3', hierarchy=('Test', 'test'), organism='3702')

        sets = GeneSets([gs1, gs2, gs3])
        self.assertIsNotNone(sets)

        self.assertRaises(GeneSetException, sets.common_org)
        self.assertRaises(GeneSetException, sets.common_hierarchy)

        self.assertGreater(len(sets.hierarchies()), 1)

        split_by_hierarchy = sets.split_by_hierarchy()
        self.assertLess(len(split_by_hierarchy), len(sets))


if __name__ == '__main__':
    unittest.main()



import unittest
import os

from tempfile import mkstemp
from orangecontrib.bioinformatics.cluster_analysis import Cluster


class TestCluster(unittest.TestCase):
    gene_names = ['HBA1', 'HBA2', 'LTB', 'TMSB4X', 'HBB']
    gene_ids = [3039, 3040, 4050, 7114, 3040]

    def test_file_name(self):
        pass



if __name__ == '__main__':
    unittest.main()



import warnings
import numpy as np

from xml.sax.saxutils import escape
from itertools import chain, starmap
from collections import defaultdict, namedtuple
from functools import singledispatch

from AnyQt.QtGui import (
    QStandardItem, QStandardItemModel,
)
from AnyQt.QtCore import (
    Qt, pyqtSignal as Signal, QItemSelection, QItemSelectionModel, QPersistentModelIndex, QSize
)
from AnyQt.QtWidgets import (
    QWidget, QListView, QVBoxLayout, QGroupBox, QComboBox, QSizePolicy
)


from Orange.data import Variable, DiscreteVariable, ContinuousVariable, StringVariable
from Orange.widgets import gui
from Orange.widgets.utils import itemmodels


@singledispatch
def standarditem_from(obj):
    item = QStandardItem(str(obj))
    item.setData(obj, Qt.UserRole)
    item.setFlags(item.flags() & ~Qt.ItemIsEditable)
    return item


@standarditem_from.register(Variable)
def standarditem_from_var(var):
    item = QStandardItem(var.name)
    _, icon = gui.attributeItem(var)
    item.setIcon(icon)
    item.setToolTip(itemmodels.Variable)


def variable_tooltip(var):
    if isinstance(var, DiscreteVariable):
        return discrete_variable_tooltip(var)
    elif isinstance(var, ContinuousVariable):
        return continuous_variable_toltip(var)
    elif isinstance(var, StringVariable):
        return string_variable_tooltip(var)


def variable_labels_tooltip(var):
    text = ""
    if var.attributes:
        items = [(escape(key), escape(value))
                 for key, value in var.attributes.items()]
        labels = list(starmap("{!s} = {!s}".format, items))
        text += "<br/>Variable Labels:<br/>"
        text += "<br/>".join(labels)
    return text


def discrete_variable_tooltip(var):
    text = "<b>%s</b><br/>Discrete with %i values: " %\
           (escape(var.name), len(var.values))
    text += ", ".join("%r" % escape(v) for v in var.values)
    text += variable_labels_tooltip(var)
    return text


def continuous_variable_toltip(var):
    text = "<b>%s</b><br/>Continuous" % escape(var.name)
    text += variable_labels_tooltip(var)
    return text


def string_variable_tooltip(var):
    text = "<b>%s</b><br/>String" % escape(var.name)
    text += variable_labels_tooltip(var)
    return text


def itemselection(modelindexlist):
    """
    Return an QtCore.QItemSelection from QModelIndex list

    Parameters
    ----------
    modelindexlist : list of QtCore.QModelIndex
        Selected model indices.

    Returns
    -------
    selection : QtCore.QItemSelection
    """
    selection = QItemSelection()
    for index in modelindexlist:
        selection.select(index, index)
    return selection


ColumnGroup = namedtuple("ColumnGroup", ["name", "key", "values"])
RowGroup = namedtuple("RowGroup", ["name", "var", "values"])


def group_candidates(data):
    items = [attr.attributes.items() for attr in data.domain.attributes]
    items = list(chain(*items))

    targets = defaultdict(set)
    for label, value in items:
        targets[label].add(str(value))

    # Need at least 2 distinct values or key
    targets = [(key, sorted(vals)) for key, vals in targets.items() if len(vals) >= 2]

    column_groups = [ColumnGroup(key, key, values)
                     for key, values in sorted(targets)]
    disc_vars = [var for var in data.domain.class_vars + data.domain.metas
                 if isinstance(var, DiscreteVariable)
                 and len(var.values) >= 2]

    row_groups = [RowGroup(var.name, var, var.values)
                  for var in disc_vars]
    return column_groups, row_groups


@standarditem_from.register(ColumnGroup)
def standarditem_from_columngroup(colgroup):
    item = QStandardItem(colgroup.name)
    item.setToolTip("Split by column label: '{!s}'".format(escape(colgroup.name)))
    item.setFlags(item.flags() & ~Qt.ItemIsEditable)
    item.setData(colgroup, Qt.UserRole)
    children = [standarditem_from(val)
                for val in colgroup.values]
    item.appendRows(children)
    return item


@standarditem_from.register(RowGroup)
def standarditem_from_rowgroup(rowgroup):
    item = QStandardItem(rowgroup.name)
    icon, _ = gui.attributeItem(rowgroup.var)
    item.setIcon(icon)
    item.setToolTip(variable_tooltip(rowgroup.var))
    item.setData(rowgroup, Qt.UserRole)
    item.setFlags(item.flags() & ~Qt.ItemIsEditable)
    children = [standarditem_from(val)
                for val in rowgroup.values]
    item.appendRows(children)
    return item


def group_selection_mask(data, group, indices):
    """ Return the selection masks for the group.
    """
    if isinstance(group, ColumnGroup):
        selected = [group.values[i] for i in indices]
        target = set([(group.key, value) for value in selected])
        I = [bool(set(var.attributes.items()).intersection(target))
             for var in data.domain.attributes]
        return np.array(I, dtype=bool)
    elif isinstance(group, RowGroup):
        target = set(indices)
        X, _ = data.get_column_view(group.var)
        I = np.zeros_like(X, dtype=bool)
        for i in target:
            I |= X == i
        return I
    else:
        raise TypeError("ColumnGroup or RowGroup expected, got {}".format(type(group).__name__))


class LabelSelectionWidget(QWidget):
    """
    A widget for selection of label values.

    The widget displays the contents of the model with two widgets:

    * The top level model items are displayed in a combo box.
    * The children of the current selected top level item are
      displayed in a subordinate list view.

    .. note:: This is not a QAbstractItemView subclass.
    """
    #: Current group/root index has changed.
    groupChanged = Signal(int)
    #: Selection for the current group/root has changed.
    groupSelectionChanged = Signal()

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.__model = None
        self.__selectionMode = QListView.ExtendedSelection

        self.__currentIndex = -1
        self.__selections = {}

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        def group_box(title):
            box = QGroupBox(title)
            box.setFlat(True)
            lay = QVBoxLayout()
            lay.setContentsMargins(0, 0, 0, 0)
            box.setLayout(lay)
            return box

        self.labels_combo = QComboBox()
        self.values_view = QListView(
            selectionMode=self.__selectionMode
        )

        self.labels_combo.currentIndexChanged.connect(
            self.__onCurrentIndexChanged)

        l_box = group_box(self.tr("Label"))
        v_box = group_box(self.tr("Values"))

        l_box.layout().addWidget(self.labels_combo)
        v_box.layout().addWidget(self.values_view)

        layout.addWidget(l_box)
        layout.addWidget(v_box)

        self.setLayout(layout)

        self.setSizePolicy(QSizePolicy.Expanding,
                           QSizePolicy.Expanding)

    def clear(self):
        """ Clear the widget/model (same as ``setModel(None)``).
        """
        if self.__model is not None:
            self.values_view.selectionModel().clearSelection()
            self.values_view.selectionModel().selectionChanged.disconnect(
                self.__onSelectionChanged)

            self.values_view.setModel(None)
            self.labels_combo.setModel(QStandardItemModel(self.labels_combo))
            self.__currentIndex = -1
            self.__selections = {}
            self.__model = None

    def setModel(self, model):
        """
        Set the source model for display.

        The model should be a tree model with depth 2.

        Parameters
        ----------
        model : QtCore.QAbstractItemModel
        """
        if model is self.__model:
            return

        self.clear()

        if model is None:
            return

        self.__model = model
        self.values_view.setModel(model)
        self.values_view.setRootIndex(model.index(0, 0))
        self.values_view.selectionModel().selectionChanged.connect(
            self.__onSelectionChanged)
        # will emit the currentIndexChanged (if the model is not empty)
        self.labels_combo.setModel(model)

    def model(self):
        """
        Return the current model.

        Returns
        -------
        model : QtCore.QAbstractItemModel
        """
        return self.__model

    def setCurrentGroupIndex(self, index):
        """
        Set the current selected group/root index row.

        Parameters
        ----------
        index : int
            Group index.
        """
        self.labels_combo.setCurrentIndex(index)

    def currentGroupIndex(self):
        """
        Return the current selected group/root index row.

        Returns
        -------
        row : index
            Current group row index (-1 if there is no current index)
        """
        return self.labels_combo.currentIndex()

    def setSelection(self, selection):
        """
        Set the model item selection.

        Parameters
        ----------
        selection : QtCore.QItemSelection
            Item selection.
        """
        if self.values_view.selectionModel() is not None:
            indices = selection.indexes()
            pind = defaultdict(list)

            for index in indices:
                parent = index.parent()
                if parent.isValid():
                    if parent == self.__model.index(parent.row(),
                                                    parent.column()):
                        pind[parent.row()].append(QPersistentModelIndex(index))
                    else:
                        warnings.warn("Die Die Die")
                else:
                    # top level index
                    pass

            self.__selections = pind
            self.__restoreSelection()

    def selection(self):
        """
        Return the item selection.

        Returns
        -------
        selection : QtCore.QItemSelection
        """
        selection = QItemSelection()
        if self.__model is None:
            return selection

        for pind in chain(*self.__selections.values()):
            ind = self.__model.index(pind.row(), pind.column(), pind.parent())
            if ind.isValid():
                selection.select(ind, ind)
        return selection

    def currentGroupSelection(self):
        """
        Return the item selection for the current group only.
        """
        if self.values_view.selectionModel() is not None:
            return self.values_view.selectionModel().selection()
        else:
            return QItemSelection()

    def __onCurrentIndexChanged(self, index):
        self.__storeSelection(self.__currentIndex,
                              self.values_view.selectedIndexes())

        self.__currentIndex = index
        if self.__model is not None:
            root = self.__model.index(index, 0)
            self.values_view.setRootIndex(root)

            self.__restoreSelection()
        self.groupChanged.emit(index)

    def __onSelectionChanged(self, old, new):
        self.__storeSelection(self.__currentIndex,
                              self.values_view.selectedIndexes())

        self.groupSelectionChanged.emit()

    def __storeSelection(self, groupind, indices):
        # Store current values selection for the current group
        groupind = self.__currentIndex
        indices = [QPersistentModelIndex(ind)
                   for ind in self.values_view.selectedIndexes()]
        self.__selections[groupind] = indices

    def __restoreSelection(self):
        # Restore previous selection for root (if available)
        assert self.__model is not None
        groupind = self.__currentIndex
        root = self.__model.index(groupind, 0)
        sel = self.__selections.get(groupind, [])
        indices = [self.__model.index(pind.row(), pind.column(), root)
                   for pind in sel if pind.isValid() and pind.parent() == root]

        selection = QItemSelection()
        for ind in indices:
            selection.select(ind, ind)
        self.values_view.selectionModel().select(
            selection, QItemSelectionModel.ClearAndSelect)

    def sizeHint(self):
        """Reimplemented from QWidget.sizeHint"""
        return QSize(100, 200)



import unittest
import scipy.stats
import numpy as np

from orangecontrib.bioinformatics.widgets.OWDifferentialExpression import f_oneway


class TestFOneWay(unittest.TestCase):
    def test_f_oneway(self):
        g1 = np.array([0.1, -0.1, 0.2, -0.2])
        g2 = g1 + 1
        g3 = g1

        f1, p1 = scipy.stats.f_oneway(g1, g2)
        f, p = f_oneway(g1, g2)
        np.testing.assert_almost_equal([f, p], [f1, p1])

        f, p = f_oneway(np.c_[g1], np.c_[g2], axis=0)
        np.testing.assert_almost_equal([f[0], p[0]], [f1, p1])

        f1, p1 = scipy.stats.f_oneway(g1, g2, g3)
        f, p = f_oneway(g1, g2, g3)
        np.testing.assert_almost_equal([f, p], [f1, p1])

        G1 = np.random.normal(size=(10, 30))
        G2 = np.random.normal(loc=1, size=(10, 20))
        G3 = np.random.normal(loc=2, size=(10, 10))

        F, P = f_oneway(G1, G2, G3, axis=1)
        self.assertEqual(F.shape, (10,))
        self.assertEqual(P.shape, (10,))

        FP1 = [scipy.stats.f_oneway(g1, g2, g3)
               for g1, g2, g3 in zip(G1, G2, G3)]

        F1 = [f for f, _ in FP1]
        P1 = [p for _, p in FP1]
        np.testing.assert_almost_equal(F1, F)
        np.testing.assert_almost_equal(P1, P)

        F, P = f_oneway(G1.T, G2.T, G3.T, axis=0)
        np.testing.assert_almost_equal(F1, F)
        np.testing.assert_almost_equal(P1, P)


from unittest import TestCase

from server_update import sf_local
from orangecontrib.bioinformatics.go import Ontology, Annotations


class GOTest(TestCase):

    def setUp(self):
        print(sf_local.listfiles('GO'))
        self.organisms = [fname.split('.')[1] for domain,temp, fname in sf_local.listfiles('GO')
                          if fname.startswith('gene_association')]
        self.onto = Ontology()

    def test_ontology(self):
        self.assertGreater(len(self.onto), 46000)

    def test_associations(self):
        for organism in self.organisms:
            Annotations(organism, ontology=self.onto)



""" GeneSets """
import threading
import concurrent.futures

from functools import partial
from collections import defaultdict

from AnyQt.QtWidgets import (
    QTreeView, QHeaderView, QHBoxLayout
)
from AnyQt.QtCore import (
    Qt, QSize, QThreadPool, Slot, QThread
)
from AnyQt.QtGui import (
    QColor, QStandardItemModel, QStandardItem,
)

from Orange.widgets.gui import (
    vBox, lineEdit, ProgressBar, LinkRole, LinkStyledItemDelegate, doubleSpin,
    auto_commit, widgetLabel, spin, comboBox, widgetBox, radioButtonsInBox
)
from Orange.data import Domain, Table, DiscreteVariable, StringVariable
from Orange.widgets.widget import OWWidget, Msg
from Orange.widgets.settings import Setting, ContextSetting
from Orange.widgets.utils.signals import Output, Input
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.widgets.utils.concurrent import ThreadExecutor, FutureWatcher, methodinvoke

from orangecontrib.bioinformatics.widgets.utils.data import (
    TAX_ID, GENE_AS_ATTRIBUTE_NAME, GENE_ID_COLUMN, GENE_ID_ATTRIBUTE,
    ERROR_ON_MISSING_ANNOTATION, ERROR_ON_MISSING_GENE_ID, ERROR_ON_MISSING_TAX_ID
)

from orangecontrib.bioinformatics.widgets.utils.gui import GeneSetsSelection
from orangecontrib.bioinformatics.widgets.utils.gui import NumericalColumnDelegate, FilterProxyModel
from orangecontrib.bioinformatics.widgets.utils.settings import OrganismContextHandler
from orangecontrib.bioinformatics.utils.statistics import FDR
from orangecontrib.bioinformatics import geneset


class Task:
    future = None
    watcher = None
    cancelled = False

    def cancel(self):
        self.cancelled = True
        self.future.cancel()
        concurrent.futures.wait([self.future])


class OWGeneSets(OWWidget):
    name = "Gene Set Enrichment"
    description = ""
    icon = "icons/OWGeneSets.svg"
    priority = 9
    want_main_area = True
    settingsHandler = OrganismContextHandler()

    # settings
    auto_commit = Setting(True)
    stored_gene_sets_selection = ContextSetting([])
    organism = ContextSetting(None)

    min_count = Setting(5)
    use_min_count = Setting(True)

    max_p_value = Setting(0.0001)
    use_p_value = Setting(False)

    max_fdr = Setting(0.01)
    use_max_fdr = Setting(True)

    use_reference_data = Setting(True)

    custom_gene_set_indicator = Setting(None)

    COUNT, REFERENCE, P_VAL, FDR, ENRICHMENT, GENES, CATEGORY, TERM = range(8)
    DATA_HEADER_LABELS = ["Count", 'Reference', 'p-Value', 'FDR', 'Enrichment', 'Genes In Set', 'Category', 'Term']

    class Inputs:
        genes = Input("Data", Table)
        custom_sets = Input('Custom Gene Sets', Table)
        reference = Input("Reference Genes", Table)

    class Outputs:
        matched_genes = Output("Matched Genes", Table)

    class Information(OWWidget.Information):
        pass

    class Warning(OWWidget.Warning):
        all_sets_filtered = Msg('All sets were filtered out.')

    class Error(OWWidget.Error):
        organism_mismatch = Msg('Organism in input data and custom gene sets does not match')
        missing_annotation = Msg(ERROR_ON_MISSING_ANNOTATION)
        missing_gene_id = Msg(ERROR_ON_MISSING_GENE_ID)
        missing_tax_id = Msg(ERROR_ON_MISSING_TAX_ID)
        cant_reach_host = Msg("Host orange.biolab.si is unreachable.")
        cant_load_organisms = Msg("No available organisms, please check your connection.")

    def __init__(self):
        super().__init__()

        # commit
        self.commit_button = None

        # progress bar
        self.progress_bar = None
        self.progress_bar_iterations = None

        # data
        self.input_data = None
        self.input_genes = []
        self.tax_id = None
        self.use_attr_names = None
        self.gene_id_attribute = None
        self.gene_id_column = None

        # custom gene sets
        self.custom_data = None
        self.feature_model = DomainModel(valid_types=(DiscreteVariable, StringVariable))
        self.gs_label_combobox = None
        self.custom_tax_id = None
        self.custom_use_attr_names = None
        self.custom_gene_id_attribute = None
        self.custom_gene_id_column = None
        self.num_of_custom_sets = None

        # Gene Sets widget
        self.gs_widget = None

        # reference genes
        self.reference_radio_box = None
        self.reference_data = None
        self.reference_genes = None

        self.reference_tax_id = None
        self.reference_attr_names = None
        self.reference_gene_id_attribute = None
        self.reference_gene_id_column = None

        # info box
        self.input_info = None
        self.num_of_sel_genes = 0

        # filter
        self.line_edit_filter = None
        self.search_pattern = ''
        self.organism_select_combobox = None

        # data model view
        self.data_view = None
        self.data_model = None

        # gene matcher NCBI
        self.gene_matcher = None

        # filter proxy model
        self.filter_proxy_model = None

        # hierarchy widget
        self.hierarchy_widget = None
        self.hierarchy_state = None

        # spinbox
        self.spin_widget = None

        # threads
        self.threadpool = QThreadPool(self)
        self.workers = None

        self._task = None  # type: Optional[Task]
        self._executor = ThreadExecutor()

        # gui
        self.setup_gui()

    def __reset_widget_state(self):
        self.update_info_box()
        # clear data view
        self.init_item_model()
        # reset filters
        self.setup_filter_model()

    def cancel(self):
        """
        Cancel the current task (if any).
        """
        if self._task is not None:
            self._task.cancel()
            assert self._task.future.done()
            # disconnect the `_task_finished` slot
            self._task.watcher.done.disconnect(self._init_gene_sets_finished)
            self._task = None

    @Slot()
    def progress_advance(self):
        # GUI should be updated in main thread. That's why we are calling advance method here
        if self.progress_bar:
            self.progress_bar.advance()

    def __get_input_genes(self):
        self.input_genes = []

        if self.use_attr_names:
            for variable in self.input_data.domain.attributes:
                self.input_genes.append(str(variable.attributes.get(self.gene_id_attribute, '?')))
        else:
            genes, _ = self.input_data.get_column_view(self.gene_id_column)
            self.input_genes = [str(g) for g in genes]

    def handle_custom_gene_sets(self, select_customs_flag=False):
        if self.custom_gene_set_indicator:
            if self.custom_data is not None and self.custom_gene_id_column is not None:

                if self.__check_organism_mismatch():
                    self.gs_label_combobox.setDisabled(True)
                    self.Error.organism_mismatch()
                    self.gs_widget.update_gs_hierarchy()
                    return

                if isinstance(self.custom_gene_set_indicator, DiscreteVariable):
                    gene_sets_names = self.custom_gene_set_indicator.values
                else:
                    gene_sets_names, _ = self.custom_data.get_column_view(self.custom_gene_set_indicator)

                self.num_of_custom_sets = len(set(gene_sets_names))
                gene_names, _ = self.custom_data.get_column_view(self.custom_gene_id_column)
                hierarchy_title = (self.custom_data.name if self.custom_data.name else 'Custom sets', )
                try:
                    self.gs_widget.add_custom_sets(
                        gene_sets_names, gene_names,
                        hierarchy_title=hierarchy_title, select_customs_flag=select_customs_flag)
                except geneset.GeneSetException:
                    pass
                self.gs_label_combobox.setDisabled(False)
            else:
                self.gs_widget.update_gs_hierarchy()

        self.update_info_box()

    def update_tree_view(self):
        if self.use_reference_data and self.reference_data:
            self.init_gene_sets(reference_genes=self.reference_genes)
        else:
            self.init_gene_sets()

    def invalidate(self):
        # clear
        self.__reset_widget_state()
        self.update_info_box()

        if self.input_data is not None:
            # setup
            self.__get_input_genes()
            self.update_tree_view()

    def __check_organism_mismatch(self):
        """ Check if organisms from different inputs match.

        :return: True if there is a mismatch
        """
        if self.tax_id is not None and self.custom_tax_id is not None:
            return self.tax_id != self.custom_tax_id
        return False

    def __get_reference_genes(self):
        self.reference_genes = []

        if self.reference_attr_names:
            for variable in self.reference_data.domain.attributes:
                self.reference_genes.append(str(variable.attributes.get(self.reference_gene_id_attribute, '?')))
        else:
            genes, _ = self.reference_data.get_column_view(self.reference_gene_id_column)
            self.reference_genes = [str(g) for g in genes]

    def _handle_future_model(self):
        if self.custom_gene_set_indicator in self.feature_model:
            index = self.feature_model.indexOf(self.custom_gene_set_indicator)
            self.custom_gene_set_indicator = self.feature_model[index]
        else:
            if self.feature_model:
                self.custom_gene_set_indicator = self.feature_model[0]
            else:
                self.custom_gene_set_indicator = None

    @Inputs.reference
    def handle_reference_genes(self, data):
        """
        Set the (optional) input dataset with reference gene names.
        """

        if data:
            self.reference_data = data
            self.reference_tax_id = str(self.reference_data.attributes.get(TAX_ID, None))
            self.reference_attr_names = self.reference_data.attributes.get(GENE_AS_ATTRIBUTE_NAME, None)
            self.reference_gene_id_attribute = self.reference_data.attributes.get(GENE_ID_ATTRIBUTE, None)
            self.reference_gene_id_column = self.reference_data.attributes.get(GENE_ID_COLUMN, None)

            if not (self.reference_attr_names is not None
                    and ((self.reference_gene_id_attribute is None) ^ (self.reference_gene_id_column is None))):

                if self.reference_tax_id is None:
                    self.Error.missing_annotation()
                    return

                self.Error.missing_gene_id()
                return

            elif self.reference_tax_id is None:
                self.Error.missing_tax_id()
                return

        self.__get_reference_genes()
        self.reference_radio_box.setEnabled(bool(self.reference_data))
        self.invalidate()

    @Inputs.custom_sets
    def handle_custom_input(self, data):
        self.Error.clear()
        self.__reset_widget_state()
        self.custom_data = None
        self.custom_tax_id = None
        self.custom_use_attr_names = None
        self.custom_gene_id_attribute = None
        self.custom_gene_id_column = None
        self.gs_label_combobox.setDisabled(True)
        self.feature_model.set_domain(None)

        if data:
            self.custom_data = data
            self.feature_model.set_domain(self.custom_data.domain)
            self.custom_tax_id = str(self.custom_data.attributes.get(TAX_ID, None))
            self.custom_use_attr_names = self.custom_data.attributes.get(GENE_AS_ATTRIBUTE_NAME, None)
            self.custom_gene_id_attribute = self.custom_data.attributes.get(GENE_ID_ATTRIBUTE, None)
            self.custom_gene_id_column = self.custom_data.attributes.get(GENE_ID_COLUMN, None)
            self._handle_future_model()

        self.gs_label_combobox.setDisabled(True)
        self.gs_widget.clear_custom_sets()
        self.handle_custom_gene_sets(select_customs_flag=True)
        self.invalidate()

    @Inputs.genes
    def handle_genes_input(self, data):
        self.closeContext()
        self.Error.clear()
        self.__reset_widget_state()
        # clear output
        self.Outputs.matched_genes.send(None)
        # clear input values
        self.input_genes = []
        self.input_data = None
        self.stored_gene_sets_selection = tuple()
        self.tax_id = None
        self.use_attr_names = None
        self.gene_id_attribute = None
        self.gs_label_combobox.setDisabled(True)
        self.gs_widget.clear()
        self.gs_widget.clear_gene_sets()
        self.update_info_box()

        if data:
            self.input_data = data
            self.tax_id = str(self.input_data.attributes.get(TAX_ID, None))
            self.use_attr_names = self.input_data.attributes.get(GENE_AS_ATTRIBUTE_NAME, None)
            self.gene_id_attribute = self.input_data.attributes.get(GENE_ID_ATTRIBUTE, None)
            self.gene_id_column = self.input_data.attributes.get(GENE_ID_COLUMN, None)
            self.update_info_box()

            if not(self.use_attr_names is not None
                   and ((self.gene_id_attribute is None) ^ (self.gene_id_column is None))):

                if self.tax_id is None:
                    self.Error.missing_annotation()
                    return

                self.Error.missing_gene_id()
                return

            elif self.tax_id is None:
                self.Error.missing_tax_id()
                return

            if self.__check_organism_mismatch():
                self.Error.organism_mismatch()
                return

            self.openContext(self.tax_id)

            self.gs_widget.load_gene_sets(self.tax_id)

            # if input data change, we need to set feature model again
            if self.custom_data:
                self.gs_widget.clear_custom_sets()
                self._handle_future_model()
                self.handle_custom_gene_sets()

            self.invalidate()

    def update_info_box(self):
        info_string = ''
        if self.input_genes:
            info_string += '{} unique gene names on input.\n'.format(len(self.input_genes))
            info_string += '{} genes on output.\n'.format(self.num_of_sel_genes)
        else:
            if self.input_data:
                if not any([self.gene_id_column, self.gene_id_attribute]):
                    info_string += 'Input data with incorrect meta data.\nUse Gene Name Matcher widget.'
            else:
                info_string += 'No data on input.\n'

        if self.custom_data:
            info_string += '{} marker genes in {} sets\n'.format(self.custom_data.X.shape[0], self.num_of_custom_sets)


        self.input_info.setText(info_string)

    def init_gene_sets(self, reference_genes=None):
        if self._task is not None:
            self.cancel()
        assert self._task is None

        self._task = Task()
        progress_advance = methodinvoke(self, "progress_advance")

        def callback():
            if self._task.cancelled:
                raise KeyboardInterrupt()
            if self.progress_bar:
                progress_advance()

        if reference_genes is None:
            reference_genes = self.gs_widget.gs_object.genes()

        self.init_item_model()

        sets_to_display = self.gs_widget.get_hierarchies(only_selected=True)
        # save setting on selected hierarchies
        self.stored_gene_sets_selection = sets_to_display
        # save context
        self.closeContext()

        f = partial(self.set_items,
                    self.gs_widget.gs_object,
                    sets_to_display,
                    set(self.input_genes),
                    reference_genes,
                    self.min_count if self.use_min_count else 1,
                    callback=callback)

        progress_iterations = sum([len(g_set) for hier, g_set
                                   in self.gs_widget.gs_object.map_hierarchy_to_sets().items()
                                   if hier in sets_to_display])

        self.progress_bar = ProgressBar(self, iterations=progress_iterations)

        self._task.future = self._executor.submit(f)

        self._task.watcher = FutureWatcher(self._task.future)
        self._task.watcher.done.connect(self._init_gene_sets_finished)

        self.openContext(self.tax_id)

    @Slot(concurrent.futures.Future)
    def _init_gene_sets_finished(self, f):
        assert self.thread() is QThread.currentThread()
        assert threading.current_thread() == threading.main_thread()
        assert self._task is not None
        assert self._task.future is f
        assert f.done()

        self._task = None
        self.progress_bar.finish()
        self.setStatusMessage('')

        try:
            results = f.result()  # type: list
            [self.data_model.appendRow(model_item) for model_item in results]
            self.filter_proxy_model.setSourceModel(self.data_model)
            self.data_view.selectionModel().selectionChanged.connect(self.commit)
            self._update_fdr()
            self.filter_data_view()
            self.update_info_box()
        except Exception as ex:
            print(ex)

    def filter_data_view(self):
        filter_proxy = self.filter_proxy_model  # type: FilterProxyModel
        model = filter_proxy.sourceModel()      # type: QStandardItemModel

        if isinstance(model, QStandardItemModel):

            search_term = self.search_pattern.lower().strip().split()

            # apply filtering rules
            filters = [
                FilterProxyModel.Filter(
                    self.TERM, Qt.DisplayRole,
                    lambda value: all(fs in value.lower() for fs in search_term))
            ]

            # if self.use_min_count:
            #    filters.append(
            #        FilterProxyModel.Filter(
            #            self.COUNT, Qt.DisplayRole,
            #            lambda value: value >= self.min_count,
            #        )
            #    )

            if self.use_p_value:
                filters.append(
                    FilterProxyModel.Filter(
                        self.P_VAL, Qt.DisplayRole,
                        lambda value: value < self.max_p_value
                    )
                )

            if self.use_max_fdr:
                filters.append(
                    FilterProxyModel.Filter(
                        self.FDR, Qt.DisplayRole,
                        lambda value: value < self.max_fdr
                    )
                )

            filter_proxy.set_filters(filters)

            if model.rowCount() and not filter_proxy.rowCount():
                self.Warning.all_sets_filtered()
            else:
                self.Warning.clear()

    def __get_source_data(self, proxy_row_index, column):
        proxy_index = self.filter_proxy_model.index(proxy_row_index, column)
        source_index = self.filter_proxy_model.mapToSource(proxy_index)
        return source_index.data(role=Qt.DisplayRole)

    def _update_fdr(self):
        # Update the FDR in place due to a changed selected categories set and
        # results for all of these categories are already available.
        proxy = self.filter_proxy_model
        model = self.filter_proxy_model.sourceModel()

        if model is not None:
            assert isinstance(model, QStandardItemModel)

            p_values = [(i, self.__get_source_data(i, self.P_VAL)) for i in range(proxy.rowCount())]
            fdr_values = FDR([p_val for _, p_val in p_values])

            for i, fdr_val in zip([i for i, _ in p_values], fdr_values):
                proxy_index = proxy.index(i, self.FDR)
                source_index = self.filter_proxy_model.mapToSource(proxy_index)
                source_item = model.item(source_index.row(), self.FDR)
                source_item.setData(fdr_val, role=Qt.DisplayRole)
                source_item.setData(fdr_val, role=Qt.ToolTipRole)

    def commit(self):
        selection_model = self.data_view.selectionModel()

        if selection_model:
            # genes_from_set = selection_model.selectedRows(GENES)
            matched_genes = selection_model.selectedRows(self.COUNT)

            if matched_genes and self.input_genes:
                genes = [model_index.data(Qt.UserRole) for model_index in matched_genes]
                output_genes = [gene_name for gene_name in list(set.union(*genes))]
                self.num_of_sel_genes = len(output_genes)
                self.update_info_box()

                if self.use_attr_names:
                    selected = [column for column in self.input_data.domain.attributes
                                if self.gene_id_attribute in column.attributes and
                                str(column.attributes[self.gene_id_attribute]) in output_genes]

                    domain = Domain(selected, self.input_data.domain.class_vars, self.input_data.domain.metas)
                    new_data = self.input_data.from_table(domain, self.input_data)
                    self.Outputs.matched_genes.send(new_data)

                else:
                    selected_rows = []
                    for row_index, row in enumerate(self.input_data):
                        gene_in_row = str(row[self.gene_id_column])
                        if gene_in_row in self.input_genes and gene_in_row in output_genes:
                            selected_rows.append(row_index)

                    if selected_rows:
                        selected = self.input_data[selected_rows]
                    else:
                        selected = None

                    self.Outputs.matched_genes.send(selected)

    def assign_delegates(self):
        self.data_view.setItemDelegateForColumn(
            self.GENES, NumericalColumnDelegate(self)
        )

        self.data_view.setItemDelegateForColumn(
            self.COUNT, NumericalColumnDelegate(self)
        )

        self.data_view.setItemDelegateForColumn(
            self.REFERENCE, NumericalColumnDelegate(self)
        )

        self.data_view.setItemDelegateForColumn(
            self.P_VAL, NumericalColumnDelegate(self, precision=2, notation='e')
        )

        self.data_view.setItemDelegateForColumn(
            self.FDR, NumericalColumnDelegate(self, precision=2, notation='e')
        )

        self.data_view.setItemDelegateForColumn(
            self.ENRICHMENT, NumericalColumnDelegate(self, precision=1)
        )

    def setup_filter_model(self):
        self.filter_proxy_model = FilterProxyModel()
        self.filter_proxy_model.setFilterKeyColumn(self.TERM)
        self.data_view.setModel(self.filter_proxy_model)

    def setup_filter_area(self):
        h_layout = QHBoxLayout()
        h_layout.setSpacing(100)
        h_widget = widgetBox(self.mainArea, orientation=h_layout)

        spin(h_widget, self, 'min_count', 0, 100,
             label='Count',
             tooltip='Minimum genes count',
             checked='use_min_count',
             callback=self.invalidate,
             callbackOnReturn=True,
             checkCallback=self.invalidate)

        doubleSpin(h_widget, self, 'max_p_value', 0.0, 1.0, 0.0001,
                   label='p-value',
                   tooltip='Maximum p-value of the enrichment score',
                   checked='use_p_value',
                   callback=self.filter_data_view,
                   callbackOnReturn=True,
                   checkCallback=self.filter_data_view
                   )

        doubleSpin(h_widget, self, 'max_fdr', 0.0, 1.0, 0.0001,
                   label='FDR',
                   tooltip='Maximum false discovery rate',
                   checked='use_max_fdr',
                   callback=self.filter_data_view,
                   callbackOnReturn=True,
                   checkCallback=self.filter_data_view
                   )

        self.line_edit_filter = lineEdit(h_widget, self, 'search_pattern')
        self.line_edit_filter.setPlaceholderText('Filter gene sets ...')
        self.line_edit_filter.textChanged.connect(self.filter_data_view)

    def on_gene_set_indicator_changed(self):
        self._handle_future_model()
        self.gs_widget.clear_custom_sets()
        self.handle_custom_gene_sets()
        self.invalidate()

    def setup_control_area(self):
        # Control area
        self.input_info = widgetLabel(
            widgetBox(self.controlArea, "Info", addSpace=True), 'No data on input.\n'
        )

        box = vBox(self.controlArea, 'Custom Gene Set Term Column')
        self.gs_label_combobox = comboBox(box, self, "custom_gene_set_indicator", sendSelectedValue=True,
                                          model=self.feature_model, callback=self.on_gene_set_indicator_changed)
        self.gs_label_combobox.setDisabled(True)

        self.reference_radio_box = radioButtonsInBox(
            self.controlArea, self, "use_reference_data", ["Entire genome", "Reference gene set (input)"],
            tooltips=["Use entire genome (for gene set enrichment)", "Use reference set of genes"],
            box="Reference", callback=self.invalidate)

        self.reference_radio_box.setEnabled(False)

        gene_sets_box = widgetBox(self.controlArea, "Gene Sets")
        self.gs_widget = GeneSetsSelection(gene_sets_box, self, 'stored_gene_sets_selection')
        self.gs_widget.hierarchy_tree_widget.itemClicked.connect(self.update_tree_view)

        self.commit_button = auto_commit(self.controlArea, self, "auto_commit", "&Commit", box=False)

    def setup_gui(self):
        # control area
        self.setup_control_area()

        # main area
        self.data_view = QTreeView()
        self.setup_filter_model()
        self.setup_filter_area()
        self.data_view.setAlternatingRowColors(True)
        self.data_view.sortByColumn(self.COUNT, Qt.DescendingOrder)
        self.data_view.setSortingEnabled(True)
        self.data_view.setSelectionMode(QTreeView.ExtendedSelection)
        self.data_view.setEditTriggers(QTreeView.NoEditTriggers)
        self.data_view.viewport().setMouseTracking(False)
        self.data_view.setItemDelegateForColumn(self.TERM, LinkStyledItemDelegate(self.data_view))

        self.mainArea.layout().addWidget(self.data_view)

        self.data_view.header().setSectionResizeMode(QHeaderView.ResizeToContents)
        self.assign_delegates()

    @staticmethod
    def set_items(gene_sets, sets_to_display, genes, ref, count_treshold, callback):
        model_items = []
        if not genes:
            return

        for gene_set in gene_sets:
            if gene_set.hierarchy not in sets_to_display:
                continue
            enrichemnt_result = gene_set.set_enrichment(ref, genes.intersection(ref))
            callback()

            if len(enrichemnt_result.query) >= count_treshold:
                category_column = QStandardItem()
                name_column = QStandardItem()
                count_column = QStandardItem()
                genes_column = QStandardItem()
                ref_column = QStandardItem()
                pval_column = QStandardItem()
                fdr_column = QStandardItem()
                enrichemnt_column = QStandardItem()

                category_column.setData(", ".join(gene_set.hierarchy), Qt.DisplayRole)
                name_column.setData(gene_set.name, Qt.DisplayRole)
                name_column.setData(gene_set.name, Qt.ToolTipRole)
                name_column.setData(gene_set.link, LinkRole)
                name_column.setForeground(QColor(Qt.blue))

                count_column.setData(len(enrichemnt_result.query), Qt.DisplayRole)
                count_column.setData(set(enrichemnt_result.query), Qt.UserRole)

                genes_column.setData(len(gene_set.genes), Qt.DisplayRole)
                genes_column.setData(set(gene_set.genes), Qt.UserRole)  # store genes to get then on output on selection

                ref_column.setData(len(enrichemnt_result.reference), Qt.DisplayRole)

                pval_column.setData(enrichemnt_result.p_value, Qt.DisplayRole)
                pval_column.setData(enrichemnt_result.p_value, Qt.ToolTipRole)

                enrichemnt_column.setData(enrichemnt_result.enrichment_score, Qt.DisplayRole)
                enrichemnt_column.setData(enrichemnt_result.enrichment_score, Qt.ToolTipRole)

                model_items.append([count_column, ref_column, pval_column, fdr_column, enrichemnt_column,
                                    genes_column, category_column, name_column])
        return model_items

    def init_item_model(self):
        if self.data_model:
            self.data_model.clear()
            self.setup_filter_model()
        else:
            self.data_model = QStandardItemModel()

        self.data_model.setSortRole(Qt.UserRole)
        self.data_model.setHorizontalHeaderLabels(self.DATA_HEADER_LABELS)

    def sizeHint(self):
        return QSize(1280, 960)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication
    app = QApplication([])
    ow = OWGeneSets()
    ow.show()
    app.exec_()



"""
A parser for DBGET database entries

"""
from __future__ import print_function

from six import StringIO


class DBGETEntryParser(object):
    r"""
    A DBGET entry parser (inspired by ``xml.dom.pulldom``).

    Example
    -------
    >>> stream = StringIO(
    ...     "ENTRY foo\n"
    ...     "NAME  foo's name\n"
    ...     "  BAR A subsection of 'NAME'\n"
    ... )
    >>> parser = DBGETEntryParser()
    >>> for event, title, contents_part in parser.parse(stream):
    ...    print(parser.EVENTS[event], title, repr(contents_part))
    ...
    ENTRY_START None None
    SECTION_START ENTRY 'foo\n'
    SECTION_END ENTRY None
    SECTION_START NAME "foo's name\n"
    SUBSECTION_START BAR "A subsection of 'NAME'\n"
    SUBSECTION_END BAR None
    SECTION_END NAME None
    ENTRY_END None None
    """
    #: Entry start events
    ENTRY_START = 0

    #: Entry end event
    ENTRY_END = 1

    #: Section start event
    SECTION_START = 2

    #: Section end event
    SECTION_END = 3

    #: Subsection start event
    SUBSECTION_START = 4

    #: Subsection end event
    SUBSECTION_END = 5

    #: Text element event
    TEXT = 6

    EVENTS = ["ENTRY_START", "ENTRY_END", 'SECTION_START',
              'SECTION_END', 'SUBSECTION_START', 'SUBSECTION_END',
              'TEXT']

    def __init__(self):
        pass

    def parse(self, stream):
        entry_offset = None
        section_title = None
        subsection_title = None
        textline_start = None

        for line in stream:
            startswith = line.startswith
            # TODO: Reorder by frequency (for faster fallthrough)
            if startswith("ENTRY"):
                # Start parsing new entry
                yield (self.ENTRY_START, None, None)
                title, rest = self._partition_section_title(line)
                entry_offset = len(line) - len(rest)
                textline_start = " " * entry_offset
                yield (self.SECTION_START, title, rest)
                yield (self.SECTION_END, title, None)

            elif startswith("///"):
                # End entry
                if subsection_title is not None:
                    # End current subsection if any
                    yield (self.SUBSECTION_END, subsection_title, None)
                    subsection_title = None

                if section_title is not None:
                    # End current section if any
                    yield (self.SECTION_END, section_title, None)
                    section_title = None

                yield (self.ENTRY_END, None, None)
                entry_offset = None
                textline_start = None

            elif not startswith(" "):
                # Start new section
                if subsection_title is not None:
                    # End current subsection if any
                    yield (self.SUBSECTION_END, subsection_title, None)
                    subsection_title = None

                if section_title is not None:
                    # End current section if any
                    yield (self.SECTION_END, section_title, None)
                    section_title = None

                title, rest = self._partition_section_title(line)
                section_title = title
                yield (self.SECTION_START, section_title, rest)

            elif startswith(textline_start):
                # A line of text
                # TODO: pass the current subsection/section title
                yield (self.TEXT, None, line[entry_offset:])

            elif startswith(" "):
                # Start a new subsection
                if subsection_title is not None:
                    # End current subsection
                    yield (self.SUBSECTION_END, subsection_title, None)
                title, rest = self._partition_subsection_title(line)
                subsection_title = title
                yield (self.SUBSECTION_START, subsection_title, rest)

        # Close any remaining sections/entries
        if subsection_title is not None:
            yield (self.SUBSECTION_END, subsection_title, None)
        if section_title is not None:
            yield (self.SECTION_END, section_title, None)
        if entry_offset is not None:
            yield (self.ENTRY_END, None, None)

    def parse_string(self, string):
        return self.parse(StringIO(string))

    def _partition_section_title(self, line):
        """
        Split the section title from the rest of the line
        """
        try:
            title, rest = line.split(" ", 1)
        except ValueError:
            # no contents, only section title
            title = line.rstrip()
            rest = ""
        rest = rest.lstrip(" ")
        return title, rest

    def _partition_subsection_title(self, line):
        """
        Split the subsection title from the rest of the line
        """
        line = line.lstrip(" ")
        return self._partition_section_title(line)



DOMAIN = "taxonomy"
FILENAME = "ncbi-taxonomy.sqlite"
TAXDUMP_URL = "http://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz"

DEFAULT_ORGANISM = '9606'

COMMON_NAMES = (
    ("6500",   "Aplysia californica"),
    ("3702",   "Arabidopsis thaliana"),
    ("9913",   "Bos taurus"),
    ("6239",   "Caenorhabditis elegans"),
    ("5476",   "Candida albicans"),
    ("3055",   "Chlamydomonas reinhardtii"),
    ("7955",   "Danio rerio"),
    ("352472", "Dictyostelium discoideum AX4"),
    ("7227",   "Drosophila melanogaster"),
    ("562",    "Escherichia coli"),
    ("11103",  "Hepatitis C virus"),
    ("9606",   "Homo sapiens"),
    ("10090",  "Mus musculus"),
    ("2104",   "Mycoplasma pneumoniae"),
    ("4530",   "Oryza sativa"),
    ("5833",   "Plasmodium falciparum"),
    ("4754",   "Pneumocystis carinii"),
    ("10116",  "Rattus norvegicus"),
    ("4932",   "Saccharomyces cerevisiae"),
    ("4896",   "Schizosaccharomyces pombe"),
    ("31033",  "Takifugu rubripes"),
    ("8355",   "Xenopus laevis"),
    ("4577",   "Zea mays")

)

SHORT_NAMES = {
    "6500":   ["aplysia"],
    "3702":   ["arabidopsis", "thaliana", "plant"],
    "9913":   ["cattle", "cow"],
    "6239":   ["nematode", "roundworm"],
    "5476":   ["thrush", "candidiasis", "candida"],
    "3055":   ["algae"],
    "7955":   ["zebrafish"],
    "352472": ["dicty", "amoeba", "slime mold"],
    "7227":   ["fly", "fruit fly", "vinegar fly"],
    "562":    ["ecoli", "coli", "bacterium"],
    "11103":  ["virus, hepatitis"],
    "9606":   ["human"],
    "10090":  ["mouse", "mus"],
    "2104":   ["bacterium", "mycoplasma"],
    "4530":   ["asian rice", "rice", "cereal", "plant"],
    "5833":   ["plasmodium", "malaria", "parasite"],
    "4754":   ["pneumonia", "fungus"],
    "10116":  ["rat", "laboratory rat"],
    "4932":   ["yeast", "baker yeast", "brewer yeast"],
    "4896":   ["yeast", "fission yeast"],
    "31033":  ["fish", "pufferfish"],
    "8355":   ["frog", "african clawed frog"],
    "4577":   ["corn", "cereal grain", "plant"]
}


COMMON_NAMES_MAPPING = dict(COMMON_NAMES)




import orangecontrib.bioinformatics.ncbi.taxonomy

for taxid in orangecontrib.bioinformatics.ncbi.taxonomy.common_taxids():
    print("%-6s %s" % (taxid, orangecontrib.bioinformatics.ncbi.taxonomy.name(taxid)))



from unittest import TestCase
from orangecontrib.bioinformatics.ncbi import taxonomy


class TaxonomyTest(TestCase):

    def setUp(self):
        self.common_ids = taxonomy.common_taxids()
        self.organisms = [(taxonomy.name(tax_id), tax_id) for tax_id in self.common_ids]
        self.taxon = taxonomy.Taxonomy()

    def test_ids_count(self):
        self.assertGreater(len(self.taxon.taxids()), 1413000)

    def test_common_organisms(self):
        for id in self.common_ids:
            # create taxon from common organisms
            self.taxon.get_entry(id)



"""  Gene Ontology module """
import os
import tarfile
import re
import sys
import six
import warnings

from collections import defaultdict
from collections import namedtuple


from orangecontrib.bioinformatics.utils import progress_bar_milestones, serverfiles, statistics
from orangecontrib.bioinformatics.ncbi import taxonomy

from orangecontrib.bioinformatics.go.config import DOMAIN, FILENAME_ANNOTATION, FILENAME_ONTOLOGY


intern = sys.intern
default_database_path = os.path.join(serverfiles.localpath(), "GO")

_CVS_REVISION_RE = re.compile(r"^(rev)?(\d+\.\d+)+$")

evidenceTypes = {
        # Experimental
        'EXP': 'Inferred from Experiment',
        'IDA': 'Inferred from Direct Assay',
        'IPI': 'Inferred from Physical Interaction',  # [with <database:protein_name>]',
        'IMP': 'Inferred from Mutant Phenotype',
        'IGI': 'Inferred from Genetic Interaction',  # [with <database:gene_symbol[allele_symbol]>]',
        'IEP': 'Inferred from Expression Pattern',
        # Computational Analysis Evidence Codes
        'ISS': 'Inferred from Sequence Similarity',  # [with <database:sequence_id>] ',
        'ISA': 'Inferred from Sequence Alignment',
        'ISO': 'Inferred from Sequence Orthology',
        'ISM': 'Inferred from Sequence Model',
        'IGC': 'Inferred from Genomic Context',
        'RCA': 'Inferred from Reviewed Computational Analysis',
        # Author Statement Evidence Codes
        'TAS': 'Traceable author statement',
        'NAS': 'Non-traceable author statement',
        # Curatorial Statement Evidence Codes
        'IC': 'Inferred by curator',
        'ND': 'No biological data available',
        # Computationally-assigned Evidence Codes
        'IEA': 'Inferred from electronic annotation',  # [to <database:id>]',
        # Obsolete Evidence Codes
        'NR': 'Not Recorded(Obsolete)'
}

evidenceDict = defaultdict(int, [(e, 2 ** i) for i, e in enumerate(evidenceTypes.keys())])

evidenceTypesOrdered = [
    'EXP',
    'IDA',
    'IPI',
    'IMP',
    'IGI',
    'IEP',
    # Computational Analysis Evidence Codes
    'ISS',
    'ISA',
    'ISO',
    'ISM',
    'IGC',
    'RCA',
    # Author Statement Evidence Codes
    'TAS',
    'NAS',
    # Curatorial Statement Evidence Codes
    'IC',
    'ND',
    # Computationally-assigned Evidence Codes
    'IEA',
    # Obsolete Evidence Codes
    'NR'
]

multiplicitySet = set(
    ["alt_id", "is_a", "subset", "synonym", "related_synonym",
     "exact_synonym", "broad_synonym", "narrow_synonym",
     "xref_analog", "xref_unknown", "relationship"])

multipleTagSet = multiplicitySet

builtinOBOObjects = ["""
[Typedef]
id: is_a
name: is_a
range: OBO:TERM_OR_TYPE
domain: OBO:TERM_OR_TYPE
definition: The basic subclassing relationship [OBO:defs]"""
,
"""[Typedef]
id: disjoint_from
name: disjoint_from
range: OBO:TERM
domain: OBO:TERM
definition: Indicates that two classes are disjoint [OBO:defs]"""
,
"""[Typedef]
id: instance_of
name: instance_of
range: OBO:TERM
domain: OBO:INSTANCE
definition: Indicates the type of an instance [OBO:defs]"""
,
"""[Typedef]
id: inverse_of
name: inverse_of
range: OBO:TYPE
domain: OBO:TYPE
definition: Indicates that one relationship type is the inverse of another [OBO:defs]"""
,
"""[Typedef]
id: union_of
name: union_of
range: OBO:TERM
domain: OBO:TERM
definition: Indicates that a term is the union of several others [OBO:defs]"""
,
"""[Typedef]
id: intersection_of
name: intersection_of
range: OBO:TERM
domain: OBO:TERM
definition: Indicates that a term is the intersection of several others [OBO:defs]"""]


class OBOObject:
    """ Represents a generic OBO object (e.g. Term, Typedef, Instance, ...)
    """
    _INTERN_TAGS = ["id", "name", "namespace", "alt_id", "is_a"]

    def __init__(self, stanza=None, ontology=None):
        self.ontology = ontology
        self._lines = []
        self.values = {}
        self.related = set()
        self.related_to = set()
        if stanza:
            self.parse_stanza(stanza)

    def parse_stanza(self, stanza):
        intern_tags = set(self._INTERN_TAGS)
        for line in stanza.splitlines():
            if ":" not in line:
                continue
            tag, rest = line.split(":", 1)
            value, modifiers, comment = "", "", ""
            if "!" in rest:
                rest, comment = rest.split("!")
            if "{" in rest:
                value, modifiers = rest.split("{", 1)
                modifiers = modifiers.strip("}")
            else:
                value = rest
            tag = intern(tag)
            value = value.strip()
            comment = comment.strip()
            if tag in intern_tags:
                value, comment = intern(value), intern(comment)
            self._lines.append((tag, value, modifiers, comment))
            if tag in multipleTagSet:
                self.values.setdefault(tag, []).append(value)
            else:
                self.values[tag] = value
        self.related = set(self.related_objects())
        self.__dict__.update(self.values)
        if "def" in self.__dict__:
            self.__dict__["def_"] = self.__dict__["def"]

    def related_objects(self):
        """Return a list of tuple pairs where the first element is relationship
        typeId and the second id of object to whom the relationship applies to.

        """
        # TODO: add other defined Typedef ids
        typeIds = [intern("is_a")]
        result = [(typeId, id) for typeId in typeIds for id in self.values.get(typeId, [])]
        result = result + [tuple(map(intern, r.split(None, 1))) for r in self.values.get("relationship", [])]
        return result

    def __repr__(self):
        """ Return a string representation of the object in OBO format
        """
        repr = "[%s]\n" % type(self).__name__
        for tag, value, modifiers, comment in self._lines:
            repr = repr + tag + ": " + value
            if modifiers:
                repr = repr + "{ " + modifiers + " }"
            if comment:
                repr = repr + " ! " + comment
            repr = repr + "\n"
        return repr

    def __str__(self):
        """ Return the OBO object id entry
        """
        return "%s: %s" % (self.id, self.name)

    def __iter__(self):
        """ Iterates over sub terms
        """
        for typeId, id in self.related_to:
            yield (typeId, self.ontology[id])


class Term(OBOObject):
    pass


class Typedef(OBOObject):
    pass


class Instance(OBOObject):
    pass


class Ontology:
    """
    :class:`Ontology` is the class representing a gene ontology.

    :param str filename:
        A filename of an .obo formated file.
    :param progress_callback:
        Optional `float -> None` function.


    Example
    --------
        >>> # Load the current ontology (downloading it if necessary)
        >>> ontology = Ontology()

        >>> term_ids = list(ontology)
        >>> term = ontology[term_ids[0]]

    """
    version = 1

    def __init__(self, filename=None, progress_callback=None):
        self.terms = {}
        self.typedefs = {}
        self.instances = {}
        self.slims_subset = set()
        self.alias_mapper = {}
        self.reverse_alias_mapper = defaultdict(set)
        self.header = ""

        if filename is not None:
            self.parse_file(filename, progress_callback)
        else:
            filename = serverfiles.localpath_download(DOMAIN, FILENAME_ONTOLOGY)
            self.parse_file(filename, progress_callback)

    @classmethod
    def load(cls, progress_callback=None):
        """
        A class method that tries to load the ontology file from
        default_database_path. It looks for a filename starting with
        'gene_ontology'. If not found it will download it.

        """
        filename = serverfiles.localpath_download(DOMAIN, FILENAME_ONTOLOGY)

        return cls(filename, progress_callback=progress_callback)

    Load = load

    def parse_file(self, file, progress_callback=None):
        """
        Parse the file. file can be a filename string or an open file like
        object. The optional progressCallback will be called with a single
        argument to report on the progress.
        """
        if isinstance(file, str):
            if os.path.isfile(file) and tarfile.is_tarfile(file):
                f = tarfile.open(file).extractfile("gene_ontology_edit.obo")
            elif os.path.isfile(file):
                f = open(file)
            elif os.path.isdir(file):
                f = open(os.path.join(file, "gene_ontology_edit.obo"))
            else:
                raise ValueError("Cannot open %r for parsing" % file)
        else:
            f = file

        data = [line.decode() if not isinstance(line, str) else line for line in f.readlines()]
        data = "".join([line for line in data if not line.startswith("!")])
        self.header = data[: data.index("[Term]")]
        c = re.compile("\[.+?\].*?\n\n", re.DOTALL)
        data = c.findall(data)

        milestones = progress_bar_milestones(len(data), 90)
        for i, block in enumerate(builtinOBOObjects + data):
            if block.startswith("[Term]"):
                term = Term(block, self)
                self.terms[term.id] = term
            elif block.startswith("[Typedef]"):
                typedef = Typedef(block, self)
                self.typedefs[typedef.id] = typedef
            elif block.startswith("[Instance]"):
                instance = Instance(block, self)
                self.instances[instance.id] = instance
            if progress_callback and i in milestones:
                progress_callback(90.0 * i / len(data))

        self.alias_mapper = {}
        self.reverse_alias_mapper = defaultdict(set)
        milestones = progress_bar_milestones(len(self.terms), 10)
        for i, (id, term) in enumerate(six.iteritems(self.terms)):
            for typeId, parent in term.related:
                self.terms[parent].related_to.add((typeId, id))
            try:
                self.alias_mapper.update([(alt_id, id)
                                          for alt_id in term.alt_id])
                self.reverse_alias_mapper[id].union_update(term.alt_id)
            except AttributeError:
                pass
            if progress_callback and i in milestones:
                progress_callback(90.0 + 10.0 * i / len(self.terms))

    def defined_slims_subsets(self):
        """
        Return a list of defined subsets in the ontology.

        :rtype: :class:`list` of :class:`str`

        """
        return [line.split()[1] for line in self.header.splitlines()
                if line.startswith("subsetdef:")]

    def named_slims_subset(self, subset):
        """
        Return all term IDs in a named `subset`.

        :param str subset: A string naming a subset in the ontology.
        :rtype: :class:`list` of :class:`str`

        .. seealso:: :func:`defined_slims_subsets`

        """
        return [id for id, term in self.terms.items()
                if subset in getattr(term, "subset", set())]

    def set_slims_subset(self, subset):
        """
        Set the `slims_subset` term subset to `subset`.

        :param set subset: A subset of GO term IDs.

        `subset` may also be a string, in which case the call is equivalent
        to ``ont.set_slims_subsets(ont.named_slims_subset(subset))``

        """
        if isinstance(subset, str):
            self.slims_subset = set(self.named_slims_subset(subset))
        else:
            self.slims_subset = set(subset)

    def slims_for_term(self, term):
        """
        Return a list of slim term IDs for `term`.

        This is a list of `most specific` slim terms to which `term` belongs.

        :param str term: Term ID.

        """
        queue = set([term])
        visited = set()
        slims = set()
        while queue:
            term = queue.pop()
            visited.add(term)
            if term in self.slims_subset:
                slims.add(term)
            else:
                queue.update(set(tid for _, tid in self[term].related) -
                             visited)
        return slims

    def extract_super_graph(self, terms):
        """
        Return all super terms of `terms` up to the most general one.

        :param list terms: A list of term IDs.

        """
        terms = [terms] if isinstance(terms, str) else terms
        visited = set()
        queue = set(terms)
        while queue:
            term = queue.pop()
            visited.add(term)
            queue.update(set(tid for _, tid in self[term].related) -
                         visited)
        return visited

    def extract_sub_graph(self, terms):
        """
        Return all sub terms of `terms`.

        :param list terms: A list of term IDs.

        """
        terms = [terms] if type(terms) == str else terms
        visited = set()
        queue = set(terms)
        while queue:
            term = queue.pop()
            visited.add(term)
            queue.update(set(tid for _, tid in self[term].related_to) -
                         visited)
        return visited

    def term_depth(self, term, cache_={}):
        """
        Return the minimum depth of a `term`.

        (length of the shortest path to this term from the top level term).

        """
        if term not in cache_:
            cache_[term] = min([self.term_depth(parent) + 1
                                for _, parent in self[term].related] or
                               [1])
        return cache_[term]

    def __getitem__(self, termid):
        """
        Return a :class:`Term` object with `termid`.

        :param str term: An id of a 'Term' in the ontology.
        :rtype: :class:`Term`

        """
        if termid in self.terms:
            return self.terms[termid]
        elif termid in self.alias_mapper:
            return self.terms[self.alias_mapper[termid]]
        else:
            raise KeyError(termid)

    def __iter__(self):
        """
        Iterate over all term ids in ontology.
        """
        return iter(self.terms)

    def __len__(self):
        """
        Return number of terms in ontology.
        """
        return len(self.terms)

    def __contains__(self, termid):
        """
        Return `True` if a term with `termid` is present in the ontology.
        """
        return termid in self.terms or termid in self.alias_mapper


annotationFields = [
    "tax_id", "gene_id", "go_id", "evidence", "qualifier", "go_term", "pubMed", "aspect"
]


_AnnotationRecordBase = namedtuple(
    "AnnotationRecord",
    annotationFields
)


class AnnotationRecord(_AnnotationRecordBase):
    """ An annotation record mapping a gene to a term.

    See ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/README for description
    if individual fields under <gene2go> section.

    """
    def __new__(cls, *args):
        if len(args) == 1 and isinstance(args[0], str):
            args = map(intern, args[0].split("\t"))
        return super(AnnotationRecord, cls).__new__(cls, *args)

    @classmethod
    def from_string(cls, string):
        """ Create an instance from a line in a annotations file format from serverfiles.
        """
        return AnnotationRecord._make(map(intern, string.strip().split('\t')))


class Annotations:
    """ :class:`Annotations` object holds the annotations.

    :param str organism:
        an organism specifier (e.g. ``'9606'``). Annotations for that organism will be loaded.

    :param ontology: :class:`Ontology` object for annotations
    :type ontology: :class:`Ontology`

    """

    def __init__(self, organism, ontology=None, progress_callback=None):
        self.ontology = ontology

        #: A dictionary mapping a gene (gene_id) to a set of all annotations of that gene.
        self.gene_annotations = defaultdict(list)

        #: A dictionary mapping a GO term id to a set of annotations that are directly annotated to that term
        self.term_anotations = defaultdict(list)

        self.all_annotations = defaultdict(list)

        self._gene_names = None
        self._gene_names_dict = None

        #: A list of all :class:`AnnotationRecords` instances.
        self.annotations = []
        self.header = ''
        self.taxid = organism

        self._ontology = None

        try:
            path = serverfiles.localpath_download(DOMAIN,
                                                  FILENAME_ANNOTATION.format(organism),
                                                  progress_callback=progress_callback)
        except FileNotFoundError:
            raise taxonomy.UnknownSpeciesIdentifier(organism)

        self._parse_file(path)

    @property
    def ontology(self):
        return self._ontology

    @ontology.setter
    def ontology(self, ontology):
        """ Set the ontology to use in the annotations mapping.
        """
        self.all_annotations = defaultdict(list)
        self._ontology = ontology

    def _ensure_ontology(self):
        if self.ontology is None:
            self.ontology = Ontology()

    def _parse_file(self, file_path):

        with open(file_path, 'r') as anno_file:
            self.header = anno_file.readline()

            for line in anno_file.readlines():
                self.add_annotation(AnnotationRecord.from_string(line))

    def add_annotation(self, a):
        """ Add a single :class:`AnotationRecord` instance to this object.
        """
        if not isinstance(a, AnnotationRecord):
            a = AnnotationRecord(a)
        if not a.gene_id or not a.go_id or a.qualifier == 'NOT':
            return

        self.gene_annotations[a.gene_id].append(a)
        self.term_anotations[a.go_id].append(a)

        self.annotations.append(a)
        self.all_annotations = defaultdict(list)

    def get_genes_with_known_annotation(self, genes):
        """ Return only genes with known annotation

        :param genes: List of genes

        """
        return {gene for gene in genes if self.gene_annotations[gene]}

    def _collect_annotations(self, go_id, visited):
        """ Recursive function collects and caches all annotations for id
        """
        if go_id not in self.all_annotations and go_id not in visited:
            if go_id in self.ontology.reverse_alias_mapper:
                annotations = [self.term_anotations.get(alt_id, [])
                               for alt_id in self.ontology.reverse_alias_mapper[go_id]] + [self.term_anotations[go_id]]
            else:
                annotations = [self.term_anotations[go_id]]  # annotations for this term alone
            visited.add(go_id)

            for typeId, child in self.ontology[go_id].related_to:
                aa = self._collect_annotations(child, visited)
                if type(aa) == set:
                    annotations.append(aa)  # if it was already reduced in get_all_annotations
                else:
                    annotations.extend(aa)
            self.all_annotations[go_id] = annotations
        return self.all_annotations[go_id]

    def get_annotations_by_go_id(self, go_id):
        """ Return a set of all annotations (instances of :obj:`AnnotationRecord`)
        for GO term `id` and all it's subterms.

        :param :class:`str` go_id: GO term id

        """
        self._ensure_ontology()
        id = self.ontology.alias_mapper.get(go_id, go_id)
        if id not in self.all_annotations or type(self.all_annotations[id]) == list:
            annot_set = set()
            for annots in self._collect_annotations(id, set()):
                annot_set.update(annots)
            self.all_annotations[id] = annot_set
        return self.all_annotations[id]

    def get_genes_by_go_term(self, go_id, evidence_codes=None):
        """ Return a list of genes annotated by specified `evidence_codes`
        to GO term 'id' and all it's subterms."

        :param str go_id: GO term id

        :param list-of-strings evidence_codes:
               List of evidence codes to consider when matching annotations to terms.

        """
        evidence_codes = set(evidence_codes or evidenceDict.keys())
        annotations = self.get_annotations_by_go_id(go_id)
        return list(set([int(ann.gene_id) for ann in annotations if ann.evidence in evidence_codes]))

    def genes(self):
        return set([ann.gene_id for ann in self.annotations])

    def get_enriched_terms(self, genes,
                           reference=None,
                           evidence_codes=None,
                           slims_only=False,
                           aspect=None,
                           prob=statistics.Binomial(),
                           use_fdr=True,
                           progress_callback=None):
        """
        Return a dictionary of enriched terms, with tuples of
        (list_of_genes, p_value, reference_count) for items and term
        ids as keys. P-Values are FDR adjusted if use_fdr is True (default).

        :param genes: List of genes
        :param reference: List of genes (if None all genes included in the annotations will be used).
        :param evidence_codes:  List of evidence codes to consider.
        :param slims_only: If `True` return only slim terms.
        :param aspect: Which aspects to use. Use all by default;
                       one of Process (biological process),
                       Function (molecular function) or Component (cellular component)
        :param prob:
        :param use_fdr:
        :param progress_callback:
        """

        all_genes = set(genes)
        
        if aspect is None:
            aspects_set = {'Process', 'Component', 'Function'}
        elif isinstance(aspect, str):
            aspects_set = {aspect}
        else:
            aspects_set = aspect

        if reference is None:
            reference = self.genes()

        evidence_codes = set(evidence_codes or evidenceDict.keys())
        annotations = [ann for gene in genes for ann in self.gene_annotations[gene]
                       if ann.evidence in evidence_codes and ann.aspect in aspects_set]

        ref_annotations = set([ann for gene in reference for ann in self.gene_annotations[gene]
                               if ann.evidence in evidence_codes and ann.aspect in aspects_set])

        annotations_dict = defaultdict(set)
        for ann in annotations:
            annotations_dict[ann.go_id].add(ann)

        self._ensure_ontology()

        if slims_only and not self.ontology.slims_subset:
            warnings.warn("Unspecified slims subset in the ontology! " "Using 'goslim_generic' subset", UserWarning)
            self.ontology.set_slims_subset('goslim_generic')

        terms = annotations_dict.keys()
        filtered_terms = [term for term in terms if term in self.ontology]

        if len(terms) != len(filtered_terms):
            term_diff = set(terms) - set(filtered_terms)
            warnings.warn("%s terms in the annotations were not found in the "
                          "ontology." % ",".join(map(repr, term_diff)), UserWarning)

        terms = self.ontology.extract_super_graph(filtered_terms)
        res = {}

        milestones = progress_bar_milestones(len(terms), 100)

        for i, term in enumerate(terms):
            if slims_only and term not in self.ontology.slims_subset:
                continue
            all_annotations = self.get_annotations_by_go_id(term).intersection(ref_annotations)
            all_annotated_genes = set([ann.gene_id for ann in all_annotations])
            mapped_genes = all_genes.intersection(all_annotated_genes)

            if len(reference) > len(all_annotated_genes):
                mapped_reference_genes = reference.intersection(all_annotated_genes)
            else:
                mapped_reference_genes = all_annotated_genes.intersection(reference)

            res[term] = ([gene for gene in mapped_genes],
                         prob.p_value(len(mapped_genes),
                                      len(reference),
                                      len(mapped_reference_genes),
                                      len(genes)),
                                      len(mapped_reference_genes))

            if progress_callback and i in milestones:
                progress_callback(100.0 * i / len(terms))

        if use_fdr:
            res = sorted(res.items(), key=lambda x: x[1][1])
            res = dict([(id, (genes, p, ref))
                        for (id, (genes, _, ref)), p in
                        zip(res, statistics.FDR([p for _, (_, p, _) in res]))])
        return res

    def get_annotated_terms(self, genes, direct_annotation_only=False, evidence_codes=None, progress_callback=None):
        """ Return all terms that are annotated by genes with evidence_codes.
        """

        genes = [genes] if type(genes) == str else genes
        genes = set([gene for gene in genes])

        evidence_codes = set(evidence_codes or evidenceDict.keys())
        annotations = [ann for gene in genes for ann in self.gene_annotations[gene] if ann.evidence in evidence_codes]

        dd = defaultdict(set)
        for ann in annotations:
            dd[ann.go_id].add(ann.gene_id)

        if not direct_annotation_only:
            self._ensure_ontology()
            terms = dd.keys()
            filtered_terms = [term for term in terms if term in self.ontology]
            if len(terms) != len(filtered_terms):
                term_diff = set(terms) - set(filtered_terms)
                warnings.warn("%s terms in the annotations were not found in the " 
                              "ontology." % ",".join(map(repr, term_diff)), UserWarning)

            terms = self.ontology.extract_super_graph(filtered_terms)
            for i, term in enumerate(terms):
                term_annotations = self.get_annotations_by_go_id(term).intersection(annotations)
                dd[term].update([ann.gene_id for ann in term_annotations])
        return dict(dd)

    def __add__(self, iterable):
        """ Return a new Annotations object with combined annotations
        """
        return Annotations([a for a in self] + [a for a in iterable],
                           ontology=self.ontology)

    def __iadd__(self, iterable):
        """ Add annotations to this instance
        """
        self.extend(iterable)
        return self

    def __contains__(self, item):
        return item in self.annotations

    def __iter__(self):
        """ Iterate over all AnnotationRecord objects in annotations
        """
        return iter(self.annotations)

    def __len__(self):
        """ Return the number of annotations
        """
        return len(self.annotations)

    def __getitem__(self, index):
        """ Return the i-th annotation record
        """
        return self.annotations[index]

    def __getslice__(self, *args):
        return self.annotations.__getslice__(*args)

    def add(self, line):
        """ Add one annotation
        """
        self.add_annotation(line)

    def append(self, line):
        """ Add one annotation
        """
        self.add_annotation(line)

    def extend(self, lines):
        """ Add multiple annotations
        """
        for line in lines:
            self.add_annotation(line)


def filter_by_p_value(terms, p_value=0.01):
    """ Filters the terms by the p-value. Assumes terms is a dict with
    the same structure as returned from get_enriched_terms.
    """
    return dict(filter(lambda x: x[1][1] <= p_value, terms.items()))


filterByPValue = filter_by_p_value


if __name__ == "__main__":
    pass



#!/usr/bin/env python
import os


from setuptools import setup, find_packages


NAME = 'Orange3-Bioinformatics'
DOCUMENTATION_NAME = 'Orange Bioinformatics'
VERSION = '3.2.2'

DESCRIPTION = 'Orange Bioinformatics add-on for Orange data mining software package.'
LONG_DESCRIPTION = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()
AUTHOR = 'Bioinformatics Laboratory, FRI UL'
AUTHOR_EMAIL = 'info@biolab.si'
URL = 'https://github.com/biolab/orange3-bioinformatics'
LICENSE = 'GPL3+'

KEYWORDS = (
    # [PyPi](https://pypi.python.org) packages with keyword "orange3 add-on"
    # can be installed using the Orange Add-on Manager
    'orange3 add-on'
)

CLASSIFIERS = (
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'Topic :: Scientific/Engineering :: Visualization',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Intended Audience :: Education',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',
    'Development Status :: 1 - Planning',
    'Programming Language :: Python :: 3 :: Only',
    'Operating System :: OS Independent'
)

PACKAGES = find_packages()
PACKAGE_DATA = {}

requirements = ['requirements.txt']

INSTALL_REQUIRES = sorted(set(
    line.partition('#')[0].strip()
    for file in (os.path.join(os.path.dirname(__file__), file)
                 for file in requirements)
    for line in open(file)
) - {''})


ENTRY_POINTS = {
    'orange3.addon': (
        'bioinformatics = orangecontrib.bioinformatics'
    ),
    'orange.widgets': (
        'Bioinformatics = orangecontrib.bioinformatics.widgets'
    ),
    'orange.canvas.help': (
        'html-index = orangecontrib.bioinformatics.widgets:WIDGET_HELP_PATH'
    )
}

NAMESPACE_PACAKGES = ["orangecontrib"]

TEST_SUITE = "orangecontrib.bioinformatics.tests.suite"

if __name__ == '__main__':
    setup(
        name=NAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        url=URL,
        classifiers=CLASSIFIERS,
        license=LICENSE,
        keywords=KEYWORDS,
        packages=PACKAGES,
        package_data=PACKAGE_DATA,
        install_requires=INSTALL_REQUIRES,
        entry_points=ENTRY_POINTS,
        namespace_packages=NAMESPACE_PACAKGES,
        test_suite=TEST_SUITE,
        include_package_data=True,
        zip_safe=False,
    )



import unittest

from orangecontrib.bioinformatics.ncbi import gene


class GeneMatcher(unittest.TestCase):

    def test_types(self):
        with self.assertRaises(TypeError):
            gene.GeneMatcher(9606)

    def test_multiple_hits_scenario(self):
        input_gene_name = 'HB1'
        organism = '9606'

        gene_matcher = gene.GeneMatcher(organism)
        gene_matcher.genes = [input_gene_name]
        gene_matcher.run_matcher()
        result = gene_matcher.genes[0]

        self.assertEqual(result.input_name, input_gene_name)
        self.assertEqual(result.type_of_match, None)
        self.assertEqual(result.ncbi_id, None)
        self.assertGreater(len(result.possible_hits), 0)

    def test_symbol_match_scenario(self):
        input_gene_name = 'SCN5A'
        ncbi_id = 6331
        organism = '9606'

        gene_matcher = gene.GeneMatcher(organism)
        gene_matcher.genes = [input_gene_name]
        gene_matcher.run_matcher()
        result = gene_matcher.genes[0]

        self.assertEqual(result.input_name, input_gene_name)
        self.assertEqual(result.type_of_match, gene._symbol)
        self.assertEqual(result.ncbi_id, ncbi_id)

        result.load_ncbi_info()
        for tag in gene.GENE_INFO_TAGS:
            self.assertIsNotNone(getattr(result, tag))

    def test_case_insensitive(self):
        organism = '10090'
        original_name = 'Pou4f1'
        ncbi_id = 18996

        gene_matcher = gene.GeneMatcher(organism)
        gene_matcher.genes = ['Pou4F1', 'pou4F1', 'POU4F1', 'pou4f1', original_name]
        gene_matcher.run_matcher()

        self.assertEqual(len([g.ncbi_id for g in gene_matcher.genes if g.ncbi_id]), 1)

        gene_matcher = gene.GeneMatcher(organism, case_insensitive=True)
        gene_matcher.genes = ['Pou4F1', 'pou4F1', 'POU4F1', 'pou4f1', original_name]
        gene_matcher.run_matcher()

        self.assertEqual(len([g.ncbi_id for g in gene_matcher.genes if g.ncbi_id]), 5)
        self.assertEqual(set([g.ncbi_id for g in gene_matcher.genes if g.ncbi_id]).pop(), ncbi_id)


class GeneInfo(unittest.TestCase):

    def test_gene_info(self):
        ncbi_id = 6331
        organism = '9606'

        gene_info_obj = gene.GeneInfo(organism)
        gene_info = gene_info_obj.get_gene_by_id(ncbi_id)

        self.assertIsNone(gene_info.input_name)

        for tag in gene.GENE_INFO_TAGS:
            self.assertIsNotNone(getattr(gene_info, tag))


if __name__ == '__main__':
    unittest.main()



""" GEO Dataset (GDS) """
import gzip
import pickle


from collections import defaultdict
from Orange.data import DiscreteVariable, ContinuousVariable, StringVariable, Domain

from orangecontrib.bioinformatics.geo.utils import (
    create_table, parse_attribute_line, to_float, spots_mean, gds_ensure_downloaded
)
from orangecontrib.bioinformatics.utils import serverfiles
from orangecontrib.bioinformatics.ncbi import taxonomy
from orangecontrib.bioinformatics.geo.config import *


class MapSpotToGene:
    __slots__ = ('spot_id', 'gene_name', 'data')

    def __init__(self, spot_id, gene_name, data):
        """ Store mapping between spot id and gene.
        """
        self.spot_id = spot_id
        self.gene_name = gene_name
        self.data = data


class GDSInfo:
    def __init__(self):
        """ Retrieve infomation about `GEO DataSets <http://www.ncbi.nlm.nih.gov/sites/GDSbrowser>`_.

        The class accesses the Orange server file that either resides on the local computer or
        is automatically retrieved from Orange server. Calls to this class do not access any NCBI's servers.

        Constructor returning the object with GEO DataSets information. The constructor
        will download GEO DataSets information file (gds_info.pickled) from Orange server,
        it will first check the local copy.

        An instance behaves like a dictionary: the keys are GEO DataSets IDs, and the dictionary values
        for is a dictionary providing various information about the particular data set.

        Example
        --------
            >>> info = GDSInfo()
            >>> list(info.keys())[:5]
            ['GDS10', 'GDS100', 'GDS1001', 'GDS1002', 'GDS1003']
            >>> info['GDS10']['title']
            'Type 1 diabetes gene expression profiling'
            >>> info['GDS10']['platform_organism']
            'Mus musculus'

        """

        path = serverfiles.localpath_download(DOMAIN, GDS_INFO_FILENAME)
        with open(path, "rb") as f:
            self.info, self.excluded = pickle.load(f,  encoding='latin1')

    def keys(self): return self.info.keys()

    def items(self): return self.info.items()

    def values(self): return self.info.values()

    def clear(self): return self.info.clear()

    def __getitem__(self, key): return self.info[key]

    def __setitem__(self, key, item): self.info[key] = item

    def __len__(self): return len(self.info)

    def __iter__(self): return iter(self.info)

    def __contains__(self, key): return key in self.info


class GDS:
    def __init__(self, gds_name, remove_unknown=None):
        """ Retrieval of a specific GEO DataSet as a :obj:`Orange.data.Table`.

        Constructor returns the object that can retrieve GEO DataSet (samples and gene expressions).
        It first checks a local cache directory if the particular data file is loaded locally,
        else it downloads it from `NCBI's GEO FTP site <ftp://ftp.ncbi.nih.gov/pub/geo/DATA/SOFT/GDS/>`_.

        :param gds_name: An NCBI's ID for the data set in the form "GDSn" where "n" is a GDS ID number.

        :param remove_unknown: Remove spots with sample profiles that include unknown values. They are removed
                               if the proportion of samples with unknown values is above the threshold set by
                               ``remove_unknown``. If None, nothing is removed.

        """

        self.gds_name = gds_name
        self.filename = serverfiles.localpath(DOMAIN, self.gds_name + '.soft.gz')
        gds_ensure_downloaded(self.gds_name)

        self.spot2gene = {}
        self.gene2spots = {}

        self.info = None
        self.gds_data = None
        self.parse_file(remove_unknown=remove_unknown)

        taxid = taxonomy.search(self.info["sample_organism"], exact=True)
        self.info["taxid"] = taxid[0] if len(taxid) == 1 else None

        self.genes = sorted(self.gene2spots.keys())
        self.spots = sorted(self.spot2gene.keys())
        self.info["gene_count"] = len(self.genes)

    def parse_file(self, remove_unknown=None):
        """ Parse GDS data file. Create self.info and self.gds_data
        """

        entity_line = '^'
        entity_attribute = '!'
        attribute_separator = '='
        data_header_description = '#'

        info_dict = {'subsets': []}
        subsets_dict = {}
        data = {}

        line_indicator = {
            '^DATABASE': {},
            '^DATASET': info_dict,
            '^SUBSET': subsets_dict
        }

        with gzip.open(self.filename, 'rt') as gds_file:
            line_type = None
            subset_id = None

            for line in gds_file:
                if line.startswith(entity_line):
                    split_line = line.split('=')
                    line_type = split_line[0].strip()

                    if line_type == '^SUBSET':
                        subset_id = split_line[1].strip()
                        line_indicator[line_type][subset_id] = {}

                elif line.startswith(entity_attribute):
                    if attribute_separator in line:
                        key, value = parse_attribute_line(line.strip())
                        if not subset_id:
                            line_indicator[line_type][key] = value
                        else:
                            line_indicator[line_type][subset_id][key] = value

                elif line.startswith(data_header_description):
                    continue

                # data table row
                else:
                    d = line.rstrip().split('\t')

                    if remove_unknown and (float(d[2:].count('null')) / len(d[2:]) > remove_unknown):
                        continue

                    try:
                        data[d[0]] = MapSpotToGene(d[0], d[1], [to_float(v) for v in d[2:]])
                        # gene to spot and spot to genes mappings
                        spot, gene = d[0:2]
                        self.spot2gene[spot] = gene
                        self.gene2spots[gene] = self.gene2spots.get(gene, []) + [spot]
                    except ValueError:
                        # header
                        info_dict['samples'] = line.rstrip().split('\t')[2:]

            for sub_id, sub_dict in subsets_dict.items():
                sub_dict['id'] = sub_id
                sub_dict['sample_id'] = sub_dict['sample_id'].split(',')
                info_dict['subsets'].append(sub_dict)

            self.info = info_dict
            self.gds_data = data

    def _sample_annotations(self, sample_type=None):
        """ Return a dictionary with sample annotation.
        """
        annotation = {}
        for a in self.info['samples']:
            annotation[a] = {}
        for info in self.info['subsets']:
            if not sample_type or info['type'] == sample_type:
                for sample in info['sample_id']:
                    annotation[sample][info['type']] = info['description']
        return annotation

    def _sample_types(self):
        """ Return a set of sample types.
        """
        return set([info["type"] for info in self.info["subsets"]])

    def _sample_to_class(self, sample_type=None):
        """ Return class values for GDS samples.
        """
        annotations = self._sample_annotations(sample_type)
        return dict([(sample, "|".join([a for t, a in sorted(ann.items())])) for sample, ann in annotations.items()])

    def _to_orange_data_table(self, report_genes=True, merge_function=spots_mean, sample_type=None, transpose=False):
        """ Convert parsed GEO format to orange, save by genes or by spots.
        """
        if transpose:  # samples in rows
            sample2class = self._sample_to_class(sample_type)
            cvalues = sorted(set(sample2class.values()))
            if None in cvalues:
                cvalues.remove(None)

            samp_ann = self._sample_annotations()

            ad = defaultdict(set)
            for d in samp_ann.values():
                for n, v in d.items():
                    ad[n].add(v)

            # auto-select sample type if there is only one
            if len(ad) == 1:
                sample_type = list(ad.keys())[0]

            classvar = DiscreteVariable(name=sample_type or "class", values=cvalues)
            spots = self.genes if report_genes else self.spots
            atts = [ContinuousVariable(name=gene) for gene in spots]

            metasvar = [DiscreteVariable(name=n, values=sorted(values))
                        for n, values in ad.items() if n != sample_type]

            X = []
            Y = []
            metas = []
            for (i, sampleid) in enumerate(self.info["samples"]):
                vals = [((merge_function([self.gds_data[spot].data[i] for spot in self.gene2spots[gene]]))
                         if report_genes else self.gds_data[gene].data[i]) for gene in spots]
                X.append(vals)
                Y.append(sample2class.get(sampleid, None))
                metas.append([samp_ann[sampleid].get(n, None) for n, _ in ad.items() if n != sample_type])

            domain = Domain(atts, classvar, metas=metasvar)
            return create_table(domain, X, Y, metas)
        else:  # genes in rows
            annotations = self._sample_annotations(sample_type)
            atts = [ContinuousVariable(name=ss) for ss in self.info["samples"]]
            for i, a in enumerate(atts):
                setattr(a, "attributes", annotations[self.info["samples"][i]])

            geneatname = "gene" if report_genes else "spot"
            metasvar = [StringVariable(geneatname)]
            nameval = self.genes if report_genes else self.spots

            X = []
            metas = []
            for g in nameval:
                if report_genes:
                    X.append(list(map(lambda *x: merge_function(x), *[self.gds_data[spot].data
                                                                      for spot in self.gene2spots[g]])))
                else:
                    X.append(self.gds_data[g].data)
            metas = [[a] for a in nameval]
            domain = Domain(atts, [], metas=metasvar)
            return create_table(domain, X, None, metas)

    def get_data(self, report_genes=True, merge_function=spots_mean, sample_type=None, transpose=False):
        """

        :param bool report_genes: Microarray spots reported in the GEO data set can either be merged according to their
                             gene ids (if True) or can be left as spots.
        :param sample_type: the type of annotation, or (if :obj:`transpose` is True) the type of class labels to be
                            included in the data set. The entire annotation of samples will be included either
                            in the class value or in the ``.attributes`` field of each data set attribute.
        :param transpose: The output table can have spots/genes in rows and samples in columns
                         (False, default) or samples in rows and  spots/genes in columns (True).

        :param merge_function:

        :return: the GEO DataSet as an :obj:`Orange.data.Table`.
        """

        return self._to_orange_data_table(merge_function=merge_function, sample_type=sample_type,
                                          transpose=transpose, report_genes=report_genes)

    def __str__(self):
        return "%s (%s), samples=%s, features=%s, genes=%s, subsets=%d" % \
              (self.info["dataset_id"],
               self.info["sample_organism"],
               self.info['sample_count'],
               self.info['feature_count'],
               self.info['gene_count'],
               len(self.info['subsets']),)



DOMAIN = 'gene_sets'
GENE_SET_ATTRIBUTES = ('gs_id', 'hierarchy', 'organism', 'name', 'genes', 'description', 'link')



# cytoband
CYTOBAND_DOWNLOAD_LINK = 'http://statweb.stanford.edu/~tibs/GSA/cytobands-stanford.gmt'

# Reactome
REACTOME_DOWNLOAD_LINK = 'http://www.reactome.org/download/current/ReactomePathways.gmt.zip'
REACTOME_FILE_NAME = 'ReactomePathways.gmt'
REACTOME_DETAIL_LINK = 'https://reactome.org/content/detail/{}'

# GO
GO_TERM_LINK = 'http://amigo.geneontology.org/amigo/term/{}'


# OMIM
OMIM_LINK = 'http://www.omim.org/entry/{}'
